@@ -61,10 +61,11 @@ static gboolean zenity_error_active;
 static gboolean zenity_info_active;
 
 /* File Selection Dialog Options */
-static gboolean zenity_file_active;
-static gboolean zenity_file_directory;
-static gboolean zenity_file_save;
-static gboolean zenity_file_confirm_overwrite;
+static gboolean       zenity_file_active;
+static gboolean       zenity_file_directory;
+static gboolean       zenity_file_save;
+static gboolean       zenity_file_confirm_overwrite;
+static GtkFileFilter *zenity_file_filter;
 
 /* List Dialog Options */
 static gboolean zenity_list_active;
@@ -87,6 +88,8 @@ static gboolean zenity_progress_auto_kill;
 
 /* Question Dialog Options */
 static gboolean zenity_question_active;
+static gchar   *zenity_question_ok_button;
+static gchar   *zenity_question_cancel_button;
 
 /* Text Dialog Options */
 static gboolean zenity_text_active;
@@ -151,7 +154,8 @@ static GOptionEntry general_options[] = {
     G_OPTION_ARG_INT,
     &zenity_general_timeout_delay,
     N_("Set dialog timeout in seconds"),
-    NULL
+    /* Timeout for closing the dialog */
+    N_("TIMEOUT")
   },
   {
     NULL
@@ -391,6 +395,15 @@ static GOptionEntry file_selection_options[] = {
     N_("Confirm file selection if filename already exists"),
     NULL
   },
+  {
+    "file-filter",
+    '\0',
+    0,
+    G_OPTION_ARG_STRING_ARRAY,
+    &zenity_file_filter,
+    N_("Sets a filename filter"),
+    N_("NAME | PATTERN1 PATTERN2 ..."),
+  },
   { 
     NULL 
   } 
@@ -476,6 +489,7 @@ static GOptionEntry list_options[] = {
     G_OPTION_ARG_STRING,
     &zenity_list_print_column,
     N_("Print a specific column (Default is 1. 'ALL' can be used to print all columns)"),
+    /* Column index number to print out on a list dialog */
     N_("NUMBER")
   },
   {
@@ -606,6 +620,24 @@ static GOptionEntry question_options[] = {
     N_("Set the dialog text"),
     N_("TEXT")
   },
+  {
+    "ok-label",
+    '\0',
+    G_OPTION_FLAG_NOALIAS,
+    G_OPTION_ARG_STRING,
+    &zenity_question_ok_button,
+    N_("Sets the label of the Ok button"),
+    N_("TEXT")
+  },
+  {
+    "cancel-label",
+    '\0',
+    G_OPTION_FLAG_NOALIAS,
+    G_OPTION_ARG_STRING,
+    &zenity_question_cancel_button,
+    N_("Sets the label of the Cancel button"),
+    N_("TEXT")
+  },
   {
     "no-wrap",
     '\0',
@@ -835,6 +867,11 @@ zenity_option_free (void) {
   if (zenity_list_hide_column)
     g_free (zenity_list_hide_column);
 
+  if (zenity_question_ok_button)
+    g_free (zenity_question_ok_button);
+  if (zenity_question_cancel_button)
+    g_free (zenity_question_cancel_button);
+
   g_option_context_free (ctx);
 }
 
@@ -954,6 +991,7 @@ zenity_file_pre_callback (GOptionContext *context,
   zenity_file_directory = FALSE;
   zenity_file_save = FALSE;
   zenity_file_confirm_overwrite = FALSE;
+  zenity_file_filter = NULL;
 
   return TRUE;
 }
@@ -1206,6 +1244,7 @@ zenity_file_post_callback (GOptionContext *context,
     results->file_data->save = zenity_file_save;
     results->file_data->confirm_overwrite = zenity_file_confirm_overwrite;
     results->file_data->separator = zenity_general_separator;
+    results->file_data->filter = zenity_file_filter;
   } else {
     if (zenity_file_directory)
       zenity_option_error (zenity_option_get_name (file_selection_options, &zenity_file_directory),
@@ -1214,6 +1253,10 @@ zenity_file_post_callback (GOptionContext *context,
     if (zenity_file_save)
       zenity_option_error (zenity_option_get_name (file_selection_options, &zenity_file_save),
                            ERROR_SUPPORT);
+
+    if (zenity_file_filter)
+      zenity_option_error (zenity_option_get_name (file_selection_options, &zenity_file_filter),
+                           ERROR_SUPPORT);
   }
     
   return TRUE;
@@ -1340,7 +1383,9 @@ zenity_question_post_callback (GOptionContext *context,
   if (results->mode == MODE_QUESTION) {
     results->msg_data->dialog_text = zenity_general_dialog_text;
     results->msg_data->mode = ZENITY_MSG_QUESTION;
-    results->msg_data->no_wrap = zenity_general_dialog_no_wrap; 
+    results->msg_data->no_wrap = zenity_general_dialog_no_wrap;
+    results->msg_data->ok_label = zenity_question_ok_button;
+    results->msg_data->cancel_label = zenity_question_cancel_button;
   }
 
   return TRUE;
